@@ -13,9 +13,10 @@ import whoosh
 from bw2analyzer import PageRank
 from bw2data.backends import ActivityDataset as AD
 from flask_httpauth import HTTPBasicAuth
-from peewee import fn
+from peewee import DoesNotExist, IntegrityError, fn
 from werkzeug.security import check_password_hash, generate_password_hash
 
+from .comments import Comment, CommentThread
 from .utils import (
     MATCH_TYPE_ABBREVIATIONS,
     get_match_types,
@@ -137,12 +138,264 @@ def get_match_type_for_source_process(node):
             return MATCH_TYPE_ABBREVIATIONS.get(label, label)
 
 
+@matchbox_app.route("/delete-all-comments", methods=["GET"])
+@auth.login_required
+def delete_all_comments():
+    config = get_config()
+    if config["role"] != "editors":
+        flask.abort(403)
+
+    Comment.delete().execute()
+    CommentThread.delete().execute()
+
+    return ""
+
+
+@matchbox_app.route("/generate-fake-comments", methods=["GET"])
+@auth.login_required
+def generate_fake_comments():
+    config = get_config()
+    if config["role"] != "editors":
+        flask.abort(403)
+
+    from faker import Faker
+
+    fake = Faker(["it_IT", "en_US", "ja_JP", "ru_RU"])
+    import random
+
+    names = [fake.name() for _ in range(8)]
+    processes = [ds.id for ds, _ in zip(bd.Database(config["proxy"]), range(20))]
+
+    for process in processes:
+        num_comments = abs(int(2 * random.normalvariate(mu=0, sigma=5)))
+        if not num_comments:
+            continue
+
+        thread = CommentThread.create(
+            name=fake.text()[:20], process_id=process, resolved=random.random() < 0.2
+        )
+
+        for index in range(num_comments):
+            Comment.create(
+                thread=thread, content=fake.text(), user=random.choice(names)
+            )
+
+    return ""
+
+
+def format_process(obj_id):
+    node = bd.get_node(id=obj_id)
+    return {
+        "id": node.id,
+        "database": node["database"],
+        "name": node["name"],
+        "unit": node["unit"],
+        "location": node["location"],
+        "product": node["reference product"],
+        "url": flask.url_for("process_detail", id=node.id),
+    }
+
+
+@matchbox_app.route("/comments/resolve-thread", methods=["POST"])
+@auth.login_required
+def comments_resolve_thread():
+    """Change `resolved` state of a comment thread. Data should be a JSON-encoded
+    POST request:
+
+    ```json
+    {
+        'thread': integer,
+        'resolved': boolean
+    }
+    ```
+    """
+    get_config()
+    content = flask.request.get_json()
+    try:
+        thread = CommentThread.get(id=int(content["thread"]))
+        thread.resolved = content["resolved"]
+        thread.save()
+        return flask.jsonify(content["resolved"])
+    except (DoesNotExist, KeyError):
+        flask.abort(400)
+
+
+@matchbox_app.route("/comments/create-thread", methods=["POST"])
+@auth.login_required
+def comments_create_thread():
+    """Create a new comment thread. Must also include first comment.
+
+    Data should be a JSON-encoded POST request:
+
+    ```json
+    {
+        'thread': {
+            'name': string,
+            'process_id': integer,
+        },
+        'comment': {
+            'content': string,
+            'user': string,
+        }
+    }
+    ```
+    """
+    get_config()
+    content = flask.request.get_json()
+    try:
+        thread = CommentThread.create(
+            name=content["name"], process_id=content["process_id"]
+        )
+        Comment.create(thread=thread, content=content["content"], user=content["user"])
+        return flask.redirect(flask.url_for("/comments/read", thread=thread.id))
+    except (IntegrityError, KeyError):
+        flask.abort(400)
+
+
+@matchbox_app.route("/comments/create-comment", methods=["POST"])
+@auth.login_required
+def comments_create_comment():
+    """Create a new comment on existing thread.
+
+    Data should be a JSON-encoded POST request:
+
+    ```json
+    {
+        'thread': integer,
+        'content': string,
+        'user': string
+    }
+    ```
+    """
+    get_config()
+    content = flask.request.get_json()
+    try:
+        thread = CommentThread.get(id=int(content["thread"]))
+        comment = Comment.create(
+            thread=thread, content=content["content"], user=content["user"]
+        )
+        return flask.jsonify(
+            {
+                "id": comment.id,
+                "thread": comment.thread_id,
+                "content": comment.content,
+                "user": comment.user,
+                "position": comment.position,
+            }
+        )
+    except (DoesNotExist, KeyError):
+        flask.abort(400)
+
+
+@matchbox_app.route("/comments/read", methods=["GET"])
+@auth.login_required
+def comments_read():
+    """
+    API to populate dynamic comments display.
+
+    Optional GET parameters.
+
+    * `user`: str. Username to filter by
+    * `reporter`: str. First reporter of comment thread
+    * `process`: int. Proxy process ID
+    * `resolved`: str, either "0" or "1". Whether comment thread is resolved or not
+    * `thread`: int. Comment thread id
+
+    JSON Return format, sorted by ascending comment thread and then comment position.
+
+    {
+        "total_threads": int,
+        "total_comments": int,
+        "comments": [
+            {
+                "id": int, 1-indexed (from db),
+                "thread": int, 1-indexed (from db),
+                "content": str,
+                "position": int, 0-indexed,
+                "user": str,
+            }
+        ],
+        "threads": [
+            {
+                "id": int, 1-indexed,
+                "name": str,
+                "reporter": str,
+                "resolved": bool,
+                "created": ISO 8601 str,
+                "modified": ISO 8601 str,
+                "process": {
+                    "id": int, 1-indexed (from db),
+                    "database": str,
+                    "name": str,
+                    "unit": str,
+                    "location": str,
+                    "product": str,
+                    "url": str, relative URL,
+                },
+            }
+        ]
+    }
+
+    """
+    get_config()
+
+    comments = Comment.select().order_by(Comment.thread_id, Comment.position)
+    threads = CommentThread.select().order_by(CommentThread.id)
+
+    user = flask.request.args.get("user")
+    reporter = flask.request.args.get("reporter")
+    process = flask.request.args.get("process")
+    resolved = flask.request.args.get("resolved")
+    thread = flask.request.args.get("thread")
+
+    if user:
+        comments = comments.where(Comment.user == user)
+        threads = threads.where(CommentThread.id << {o.thread_id for o in comments})
+    if reporter:
+        thread_ids = {o.id for o in threads if o.reporter == reporter}
+        threads = threads.where(CommentThread.id << thread_ids)
+        comments = comments.where(Comment.thread_id << thread_ids)
+    if process:
+        threads = threads.where(CommentThread.process_id == int(process))
+        comments = comments.where(Comment.thread_id << {o.id for o in threads})
+    if resolved and resolved in "01":
+        threads = threads.where(
+            CommentThread.resolved == (True if resolved == "1" else False)
+        )
+        comments = comments.where(Comment.thread_id << {o.id for o in threads})
+    if thread:
+        comments = comments.where(Comment.thread_id == int(thread))
+        threads = threads.where(CommentThread.id == int(thread))
+
+    threads = [
+        {
+            "id": o.id,
+            "name": o.name,
+            "reporter": o.reporter,
+            "resolved": o.resolved,
+            "created": o.created,
+            "modified": o.modified,
+            "process": format_process(o.process_id),
+        }
+        for o in threads
+    ]
+
+    payload = {
+        "total_threads": CommentThread.select().count(),
+        "total_comments": Comment.select().count(),
+        "comments": list(comments.dicts()),
+        "threads": threads,
+    }
+    return flask.jsonify(payload)
+
+
 @matchbox_app.route("/page-rank", methods=["GET"])
 @auth.login_required
 def build_page_rank_cache():
     config = get_config()
     if config["role"] != "editors":
         flask.abort(403)
+
     pr = PageRank(bd.Database(config["source"])).calculate()
     with open(config["output_dir"] / "page-rank.json", "w") as f:
         json.dump({"scores": pr}, f, indent=2)
@@ -218,6 +471,15 @@ def maybe_int(value):
         return int(value)
 
 
+def get_proxy_url(ds):
+    if isinstance(ds, AD) and "proxy_id" in ds.data:
+        return flask.url_for("process_detail", id=ds.data["proxy_id"])
+    elif not isinstance(ds, AD) and "proxy_id" in ds:
+        return flask.url_for("process_detail", id=ds["proxy_id"])
+    else:
+        return None
+
+
 @matchbox_app.route("/processes", methods=["GET"])
 @auth.login_required
 def processes():
@@ -282,9 +544,7 @@ def processes():
         limit = limit or 250
         offset = offset or 0
         qs = [bd.get_node(id=id_)._document for _, id_ in pr[offset : offset + limit]]
-        print(limit, offset, len(qs))
         qs = apply_filter_to_qs(qs, filter_arg)
-        print(limit, offset, len(qs))
         total_records = len(bd.Database(config["source"]))
     else:
         qs = AD.select().where(AD.database == database_label)
@@ -312,12 +572,14 @@ def processes():
             {
                 "details_url": flask.url_for("process_detail", id=obj.id),
                 "match_url": flask.url_for("match", source=obj.id),
+                "proxy_url": get_proxy_url(obj),
                 "matched": bool(obj.data.get("matched")),
                 "match_type": get_match_type_for_source_process(obj),
                 "waitlist": bool(obj.data.get("waitlist")),
                 "id": obj.id,
                 "name": obj.name,
                 "location": obj.location,
+                "product": obj.product,
                 "unit": obj.data["unit"],
             }
             for obj in qs
@@ -407,9 +669,20 @@ def get_authors(authors):
         return ", ".join({obj.get("name", "Unknown") for obj in authors.values()})
 
 
-@matchbox_app.route("/delete-proxy/<id>", methods=["GET"])
+@matchbox_app.route("/remove-match/<id>", methods=["GET"])
 @auth.login_required
-def delete_proxy(id):
+def remove_match(id):
+    """Remove match information.
+
+    If the referenced process is in the proxy database, it will be deleted, and the
+    source process matching attributes will be removed.
+
+    If the reference process is in the source database, its matching attributes will
+    be removed, and it's proxy process (if any) will be deleted.
+
+    If the reference process is from any other database, no action is taken.
+
+    Returns a redirect to the source dataset process details view."""
     config = get_config()
     if isinstance(config, flask.Response):
         return config
@@ -418,26 +691,46 @@ def delete_proxy(id):
         flask.abort(403)
 
     node = bd.get_node(id=id)
-    if not node["database"] == config["proxy"]:
-        raise ValueError(
-            "Not a proxy dataset in the proxy database {}".format(config["proxy"])
+    if node["database"] == config["proxy"]:
+        if node.get("original_id"):
+            reference = bd.get_node(id=node["original_id"])
+            for field in ("proxy_id", "matched", "match_type"):
+                if field in reference:
+                    del reference[field]
+            reference.save()
+        else:
+            reference = None
+
+        node.delete()
+
+        return_url = (
+            flask.url_for("process_detail", id=reference.id)
+            if reference
+            else flask.url_for("index")
         )
-
-    if node.get("original_id"):
-        reference = bd.get_node(id=node["original_id"])
+    elif node["database"] == config["source"]:
+        if "proxy_id" in node:
+            bd.get_node(id=node["proxy_id"]).delete()
         for field in ("proxy_id", "matched", "match_type"):
-            if field in reference:
-                del reference[field]
-        reference.save()
+            if field in node:
+                del node[field]
+        node.save()
+        return_url = flask.url_for("process_detail", id=node.id)
     else:
-        reference = None
+        return_url = flask.url_for("process_detail", id=reference.id)
 
-    node.delete()
+    return flask.redirect(return_url)
 
-    if reference is None:
-        return flask.redirect(flask.url_for("index"))
+
+def get_match_type(node, proxy, mapping):
+    if "match_type" in node:
+        return mapping.get(node["match_type"], "Unknown")
+    elif proxy and "match_type" in proxy:
+        return mapping.get(proxy["match_type"], "Unknown")
+    elif node.get("matched"):
+        return "No direct match available"
     else:
-        return flask.redirect(flask.url_for("process_detail", id=reference.id))
+        return None
 
 
 @matchbox_app.route("/process/<id>", methods=["GET"])
@@ -467,7 +760,9 @@ def process_detail(id):
 
     return flask.render_template(
         "process_detail.html",
-        title="bw_matchbox Detail Page",
+        title="MB {} {} {}".format(
+            node["name"][:20], node["location"][:5], node["unit"][:5]
+        ),
         ds=node,
         config=config,
         authors=get_authors(node.get("authors")),
@@ -478,7 +773,9 @@ def process_detail(id):
         same_name=same_name,
         technosphere=technosphere,
         biosphere=biosphere,
-        match_type=matchbox_app.config["mb_match_types"].get(node.get("match_type")),
+        match_type=get_match_type(
+            node, proxy_node, matchbox_app.config["mb_match_types"]
+        ),
         total_consumers=len(node.upstream()),
         consumers=list(node.upstream())[:50],
         is_proxy=is_proxy,
@@ -524,6 +821,9 @@ def match(source):
 
     node = bd.get_node(id=source)
 
+    if node.get("matched"):
+        return flask.redirect(flask.url_for("process_detail", id=node.id))
+
     matches = bd.Database(config["target"]).search(
         node["name"] + " " + node.get("location", "")
     )
@@ -565,6 +865,23 @@ def check_similar(node, candidates):
             )
 
 
+@matchbox_app.route("/comments", methods=["GET"])
+@auth.login_required
+def comments():
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
+
+    base_url = flask.request.base_url.replace(flask.request.path, "")
+
+    return flask.render_template(
+        "comments.html",
+        title="Comments Dashboard",
+        config=config,
+        base_url=base_url,
+    )
+
+
 @matchbox_app.route("/compare/<source>/<target>", methods=["GET"])
 @auth.login_required
 def compare(source, target):
@@ -616,7 +933,6 @@ def compare(source, target):
         check_similar(obj, source_technosphere)
 
     base_url = flask.request.base_url.replace(flask.request.path, "")
-    print(base_url)
 
     return flask.render_template(
         "compare.html",
